/*==============================================================================*/
/*  Copyright (C) 2009-2018, Heartland Data inc. All Rights Reserved.           */
/*                                                                              */
/*  Title  :   4bit GPIO Buffer Driver                                          */
/*  EventID:   Via the driver                                                   */
/*  AppVer :   DT10 Ver10.XX`                                                  */
/*  FileID :   12C1                                                             */
/*  Version:   1.4                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Please customize the code for your environment.                             */
/*==============================================================================*/

/*------------------------------------------------------------------------------*/
/*  Desc:  Header for Port Controll                                             */
/*------------------------------------------------------------------------------*/
#include "Common.h"

/*------------------------------------------------------------------------------*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT10 Project setting.    */
/*------------------------------------------------------------------------------*/
#define DT_UINT unsigned int

/*------------------------------------------------------------------------------*/
/*  Macro:  DT_INLINE                                                           */
/*  Desc:   Please use "static" instead of "inline" if "inline" cannot be used. */
/*------------------------------------------------------------------------------*/
#define DT_INLINE  inline
//#define  DT_INLINE  static

/*------------------------------------------------------------------------------*/
/*  Macro:  DT_ADD_TIME_INFO                                                    */
/*  Desc:   Please set 1 if use TARGET SYSTEM TIME information with Test Point. */
/*------------------------------------------------------------------------------*/
#define DT_ADD_TIME_INFO 1

/*------------------------------------------------------------------------------*/
/*  Macro:  DT_ADD_EVENT_INFO                                                   */
/*  Desc:  Please set 1 if use EVENT TRACE DATA with Test Point.                */
/*------------------------------------------------------------------------------*/
#define DT_ADD_EVENT_INFO 0

/*------------------------------------------------------------------------------*/
/*  Desc:   Please set Buffer Size and Option Setthings                         */
/*------------------------------------------------------------------------------*/
#define RING_BUFF_USE   1                              /* Use RingBuffer */
#define REPORT_COUNT    100                            /* TP's recode Line num */

#define DT_TP_SIZE      5                              /* One TP size(addr + dat size) */

#if DT_ADD_TIME_INFO
#define DT_TIME_INFO_SIZE  4                           /* time log size */
#else
#define DT_TIME_INFO_SIZE  0                           /* time log size(not use) */
#endif /* DT_ADD_TIME_INFO */

#if DT_ADD_EVENT_INFO
#define DT_ADD_EVENT_INFO_SIZE 4                       /* Eventdata info size When the Driver support Type*/
#else
#define DT_ADD_EVENT_INFO_SIZE 0                       /* Eventdata info size */
#endif /* DT_ADD_EVENT_INFO */

#define DT_TP_MAX_SIZE  (DT_TP_SIZE + DT_TIME_INFO_SIZE+DT_ADD_EVENT_INFO_SIZE) /* One all TP's Recode  size */
#define BUFF_SIZE       (DT_TP_MAX_SIZE * REPORT_COUNT)                         /* Use Buffer Size */

static char dt_buf[BUFF_SIZE];
static unsigned long currPos;                          /* RingBuffer's current position */
#if RING_BUFF_USE
static char buf_full_flag = 0;                         /* Round RingBuffe flag */
#endif /* RING_BUFF_USE */
static char data_out = 0;

/*==============================================================================*/

/*------------------------------------------------------------------------------*/
/*  Func:   _TP_BusPortInit                                                     */
/*  Desc:   Please describe the code to initializes ports.                      */
/*------------------------------------------------------------------------------*/
static void _TP_BusPortInit(void)
{
    SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOA );
    SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOD );
    GPIOPinTypeGPIOOutput( GPIO_PORTD_BASE, GPIO_PIN_2 );   /* CLK */
    GPIOPinTypeGPIOOutput( GPIO_PORTD_BASE, GPIO_PIN_3 );   /* CS */
    GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, GPIO_PIN_2 );   /* DAT0 */
    GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, GPIO_PIN_3 );   /* DAT1 */
    GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, GPIO_PIN_4 );   /* DAT2 */
    GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, GPIO_PIN_5 );   /* DAT3 */
    GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, GPIO_PIN_2);
    GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_3, GPIO_PIN_3);
    GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_2, GPIO_PIN_2 );
    GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_3, GPIO_PIN_3 );
    GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_4, GPIO_PIN_4 );
    GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_5, GPIO_PIN_5 );
}

/*------------------------------------------------------------------------------*/
/*  Func:   portReverseCLK                                                      */
/*  Desc:   Please describe the code to reverse CLK Port.                       */
/*------------------------------------------------------------------------------*/
DT_INLINE void portReverseCLK(void)
{
    if( GPIOPinRead( GPIO_PORTD_BASE, GPIO_PIN_2 ) ){
        GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, 0 );
    }
    else{
        GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, GPIO_PIN_2 );
    }
}

/*------------------------------------------------------------------------------*/
/*  Func:   portSetCS                                                           */
/*  Desc:   Please describe the code to set the value to CS Port.               */
/*------------------------------------------------------------------------------*/
DT_INLINE void portSetCS(DT_UINT dat)
{
    GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_3, dat?0xff:0x00 );
}

/*------------------------------------------------------------------------------*/
/*  Func:   portSetDATA                                                         */
/*  Desc:   Please describe the code to set the value to DATA Port              */
/*------------------------------------------------------------------------------*/
DT_INLINE void portSetDATA(DT_UINT dat)
{
    GPIOPinWrite( GPIO_PORTA_BASE, (GPIO_PIN_2|GPIO_PIN_3|GPIO_PIN_4|GPIO_PIN_5), (dat & 0x0f) << 2 );
}

/*------------------------------------------------------------------------------*/
/*  Func:   enterCritical                                                       */
/*  Desc:   Please describe the code to enter Critical Section.                 */
/*------------------------------------------------------------------------------*/
DT_INLINE void enterCritical(void)
{
    taskENTER_CRITICAL();
}

/*------------------------------------------------------------------------------*/
/*  Func:   exitCritical                                                        */
/*  Desc:   Please describe the code to exit Critical Section.                  */
/*------------------------------------------------------------------------------*/
DT_INLINE void exitCritical(void)
{
    taskEXIT_CRITICAL();
}

/*==============================================================================*/
/*  Don't change the code from here as possible.                                */
/*==============================================================================*/

/*------------------------------------------------------------------------------*/
/*  Desc:   Test Point IDs                                                      */
/*------------------------------------------------------------------------------*/
#define     DT_TIME_INFO_BIT        0x04
#define     DT_EVTTRG_BIT           0x08
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Func:   portInit                                                            */
/*  Desc:   Initialize Port Function                                            */
/*------------------------------------------------------------------------------*/
static int init = 0;
DT_INLINE void portInit(void)
{
    unsigned long i;
    if( init == 0 ){
        _TP_BusPortInit();
        
        for(i=0;i<BUFF_SIZE;i++){
            dt_buf[i] = 0;
        }
#if RING_BUFF_USE
        buf_full_flag = 0;
#endif
        currPos = 0;
        init = 1;
    }
}

/*------------------------------------------------------------------------------*/
/*  Func: _TP_Bus4BitOutDrv                                                     */
/*  Desc: 4bit Data Output Function                                             */
/*------------------------------------------------------------------------------*/
DT_INLINE void _TP_Bus4BitOutDrv( DT_UINT dat )
{
    portSetDATA( dat );
    portReverseCLK();
}

/*------------------------------------------------------------------------------*/
/*  Func: _TP_BusOut                                                            */
/*  Desc: Called by Test Point                                                  */
/*------------------------------------------------------------------------------*/
void _TP_BusOut( DT_UINT addr, DT_UINT dat )
{
#if DT_ADD_TIME_INFO
    static unsigned long time = 0;
#endif /* DT_ADD_TIME_INFO */
#if DT_ADD_EVENT_INFO
    unsigned int event_id = 0;
#endif /* DT_ADD_EVENT_INFO */

	if (data_out == 1) {
		return;
	}
    if (currPos >= BUFF_SIZE) {
#if RING_BUFF_USE
        buf_full_flag = 1;
        currPos = 0;
#else
		return;
#endif /* RING_BUFF_USE */
    }

	enterCritical();
	if(init == 0){
		portInit();
	}
#if DT_ADD_TIME_INFO
    addr |= DT_TIME_INFO_BIT;
#endif /* DT_ADD_TIME_INFO */

#if DT_ADD_EVENT_INFO
    addr |= DT_EVTTRG_BIT;
#endif /* DT_ADD_EVENT_INFO */

    dt_buf[currPos++] = (unsigned char)((dat >> 8) & 0xFF); /* dat Upperbyte */
    dt_buf[currPos++] = (unsigned char)(dat & 0xFF); /* dat Lower1byte */

    dt_buf[currPos++] = (unsigned char)((addr >> 16) & 0xFF);   /* adrr Upper1byte */
    dt_buf[currPos++] = (unsigned char)((addr >> 8) & 0xFF);    /* adrr Lower2byte */
    dt_buf[currPos++] = (unsigned char)(addr & 0xFF);    /* adrr Lower1byte */
    
    #if DT_ADD_TIME_INFO
    /* Please customize here When if get time Information */
    time++;

    dt_buf[currPos++] = (unsigned char)(time & 0xFF);   /* time Lower1byte */
    dt_buf[currPos++] = (unsigned char)((time >> 8) & 0xFF);   /* time Lower2byte */
    dt_buf[currPos++] = (unsigned char)((time >> 16) & 0xFF);  /* time Upper1byte */
    dt_buf[currPos++] = (unsigned char)((time >> 24) & 0xFF);  /* time Upper2byte */
#endif /* DT_ADD_TIME_INFO */

#if DT_ADD_EVENT_INFO
    /* Please customize here When if get Event Information */
    event_id = 0;

    dt_buf[currPos++] = (unsigned char)((event_id >> 0) & 0xFF);   /* EventInfo Lower1byte */
    dt_buf[currPos++] = (unsigned char)((event_id >> 8) & 0xFF);   /* EventInfo Lower2byte */
    dt_buf[currPos++] = (unsigned char)((event_id >> 16) & 0xFF);  /* EventInfo Upper1byte */
    dt_buf[currPos++] = (unsigned char)((event_id >> 24) & 0xFF);  /* EventInfo Upper2byte */
#endif /* DT_ADD_EVENT_INFO */
    exitCritical();
}

/*==============================================================================*/

/*------------------------------------------------------------------------------*/
/*  Func: _TP_DataOut                                                           */
/*  Desc: Called by Output Test Point                                           */
/*------------------------------------------------------------------------------*/
extern void _TP_DataOut(void)
{
    unsigned int i, j, count;

	data_out = 1;
    enterCritical();
	count = currPos;
#if RING_BUFF_USE
	if (buf_full_flag == 1) {
		for (i = count; i < BUFF_SIZE; ) {
			/* Send TestPoint Data */
		    portSetCS(0);
			for (j = 0; j < 2; j++) {
				_TP_Bus4BitOutDrv(dt_buf[i] >> 4);
				_TP_Bus4BitOutDrv(dt_buf[i]);
				i++;
			}
			for (j = 0; j < 3; j++) {
				if (dt_buf[i] >= 0x10) _TP_Bus4BitOutDrv(dt_buf[i] >> 4);
				if (dt_buf[i] >= 0x01) _TP_Bus4BitOutDrv(dt_buf[i]);
				i++;
			}
		    portSetCS(1);
			/* Send Time Data */
#if DT_ADD_TIME_INFO
	        portSetCS(0);
			for (j = 0; j < 4; j++) {
				_TP_Bus4BitOutDrv(dt_buf[i] >> 4);
				_TP_Bus4BitOutDrv(dt_buf[i]);
				i++;
			}
#endif
			/* Send Event Data */
#if DT_ADD_EVENT_INFO
	        portSetCS(0);
			for (j = 0; j < 4; j++) {
				_TP_Bus4BitOutDrv(dt_buf[i] >> 4);
				_TP_Bus4BitOutDrv(dt_buf[i]);
				i++;
			}
#endif
	        portSetCS(1);
		}
	}
#endif /* RING_BUFF_USE */
	for (i = 0; i < count; ) {
		/* Send TestPoint Data */
	    portSetCS(0);
		for (j = 0; j < 2; j++) {
			_TP_Bus4BitOutDrv(dt_buf[i] >> 4);
			_TP_Bus4BitOutDrv(dt_buf[i]);
			i++;
		}
		for (j = 0; j < 3; j++) {
			if (dt_buf[i] >= 0x10) _TP_Bus4BitOutDrv(dt_buf[i] >> 4);
			if (dt_buf[i] >= 0x01) _TP_Bus4BitOutDrv(dt_buf[i]);
			i++;
		}
	    portSetCS(1);
		/* Send Time Data */
#if DT_ADD_TIME_INFO
        portSetCS(0);
		for (j = 0; j < 4; j++) {
			_TP_Bus4BitOutDrv(dt_buf[i] >> 4);
			_TP_Bus4BitOutDrv(dt_buf[i]);
			i++;
		}
#endif
		/* Send Event Data */
#if DT_ADD_EVENT_INFO
        portSetCS(0);
		for (j = 0; j < 4; j++) {
			_TP_Bus4BitOutDrv(dt_buf[i] >> 4);
			_TP_Bus4BitOutDrv(dt_buf[i]);
			i++;
		}
#endif
        portSetCS(1);
	}

    init = 0;    /* On Buffer Clear flag */
	data_out = 0;
    exitCritical();
}

/*==============================================================================*/
